﻿using Microsoft.Extensions.CommandLineUtils;
using System;
using System.Linq;

namespace KellermanSoftware.NetSFtpLibraryCoreDemo
{
    internal sealed class CommandLineParser
    {
        private readonly Action<string, Configuration> _configureAction;
        private readonly Action<string, string, string> _uploadAction;
        private readonly Action<string, string, string> _downloadAction;

        public CommandLineParser(Action<string, Configuration> configureAction,
                                 Action<string, string, string> uploadAction,
                                 Action<string, string, string> downloadAction)
        {
            _configureAction = configureAction;
            _uploadAction = uploadAction;
            _downloadAction = downloadAction;
        }

        public void Parse(string[] args)
        {
            try
            {
                var application = new CommandLineApplication();
                application.HelpOption("-? | -h | --help");
                SetupConfigureCommand(application);
                SetupUploadCommand(application);
                SetupDownloadCommand(application);
                int result = application.Execute(args);
                if (result == 0)
                {
                    application.ShowHelp();
                }
            }
            catch (CommandParsingException exception)
            {
                Console.WriteLine(exception.Message);
            }
        }

        private void SetupConfigureCommand(CommandLineApplication application)
        {
            CommandOption configOption = null;
            CommandOption serverOption = null;
            CommandOption usernameOption = null;
            CommandOption passwordOption = null;
            CommandOption licenseOption = null;
            CommandOption licenseUsernameOption = null;
            var command = application.Command("configure",
                                              target =>
                                              {
                                                  configOption = target.Option("-config | --config",
                                                                               "Configuration file",
                                                                               CommandOptionType.SingleValue);
                                                  serverOption = target.Option("-server | --server",
                                                                               "SFTP server address",
                                                                               CommandOptionType.SingleValue);
                                                  usernameOption = target.Option("-username | --username",
                                                                                 "User name",
                                                                                 CommandOptionType.SingleValue);
                                                  passwordOption = target.Option("-password | --password",
                                                                                 "User password",
                                                                                 CommandOptionType.SingleValue);
                                                  licenseOption = target.Option("-license | --license",
                                                                                "License key",
                                                                                CommandOptionType.SingleValue);
                                                  licenseUsernameOption = 
                                                      target.Option("-licenseusername | --licenseusername",
                                                                    "License username",
                                                                    CommandOptionType.SingleValue);
                                              });
            command.Description = "Create configuration file";
            command.HelpOption("-? | -h | --help");
            command.OnExecute(() =>
            {
                if (CheckHaveValues(configOption, 
                                    serverOption, 
                                    usernameOption, 
                                    passwordOption, 
                                    licenseOption,
                                    licenseUsernameOption))
                {
                    var configuration = new Configuration
                    {
                        Server = serverOption.Value(),
                        Username = usernameOption.Value(),
                        Password = passwordOption.Value(),
                        LicenseKey = licenseOption.Value(),
                        LicenseUsername = licenseUsernameOption.Value()
                    };
                    _configureAction(configOption.Value(), configuration);
                }
                else
                {
                    command.ShowHelp();
                }
                return 1;
            });
        }

        private void SetupUploadCommand(CommandLineApplication application)
        {
            CommandOption configOption = null;
            CommandOption sourceOption = null;
            CommandOption destinationOption = null;
            var command = application.Command("upload",
                                              target =>
                                              {
                                                  configOption = target.Option("-config | --config",
                                                                               "Configuration file",
                                                                               CommandOptionType.SingleValue);
                                                  sourceOption = target.Option("-source | --source",
                                                                               "Local source path",
                                                                               CommandOptionType.SingleValue);
                                                  destinationOption = target.Option("-dest | --dest",
                                                                                    "SFTP destination path",
                                                                                    CommandOptionType.SingleValue);
                                              });
            command.Description = "Upload file or directory from local source to SFTP destination";
            command.HelpOption("-? | -h | --help");
            command.OnExecute(() =>
            {
                if (CheckHaveValues(configOption, sourceOption, destinationOption))
                {
                    _uploadAction(configOption.Value(), sourceOption.Value(), destinationOption.Value());
                }
                else
                {
                    command.ShowHelp();
                }
                return 1;
            });
        }

        private void SetupDownloadCommand(CommandLineApplication application)
        {
            CommandOption configOption = null;
            CommandOption sourceOption = null;
            CommandOption destinationOption = null;
            var command = application.Command("download",
                                              target =>
                                              {
                                                  configOption = target.Option("-config | --config",
                                                                               "Configuration file",
                                                                               CommandOptionType.SingleValue);
                                                  sourceOption = target.Option("-source | --source",
                                                                               "SFTP source path",
                                                                               CommandOptionType.SingleValue);
                                                  destinationOption = target.Option("-dest | --dest",
                                                                                    "Local destination path",
                                                                                    CommandOptionType.SingleValue);
                                              });
            command.Description = "Download file or directory from SFTP source to local destination";
            command.HelpOption("-? | -h | --help");
            command.OnExecute(() =>
            {
                if (CheckHaveValues(configOption, sourceOption, destinationOption))
                {
                    _downloadAction(configOption.Value(), sourceOption.Value(), destinationOption.Value());
                }
                else
                {
                    command.ShowHelp();
                }
                return 1;
            });
        }

        private static bool CheckHaveValues(params CommandOption[] options)
        {
            return options.All(option => option.HasValue());
        }
    }
}
