﻿#Region "Includes"
Imports System
Imports System.Collections.Generic
Imports System.Windows.Forms
Imports System.IO
Imports KellermanSoftware.NetSFtpLibrary
#End Region

Namespace NetSFtpLibrarySimpleDemo
	Partial Public Class Demo
		Inherits Form

		#Region "Class Variables"
		Private _sftp As SFTP
		Private _files As List(Of FTPFileInfo)
		Private Delegate Sub SetStatusDelegate(ByVal text As String)
		Private Delegate Sub LoadDirectoryDelegate(ByVal path As String)
		Private Delegate Sub SetCommandGuiDelegate(ByVal enabled As Boolean)
		#End Region

		#Region "Constructor"
		Public Sub New()
			InitializeComponent()
		End Sub
		#End Region

		#Region "Methods"

		''' <summary>
		''' Show an error message box
		''' </summary>
		''' <param name="myException"></param>
		Private Sub ShowError(ByVal myException As Exception)
			MessageBox.Show(myException.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
		End Sub
		''' <summary>
		''' Event that is fired when a transfer completes
		''' </summary>
		Private Sub FtpTransferCompleted(ByVal sender As Object, ByVal e As TransferCompletedEventArgs)
			SetStatus("File transfer completed...")
			LoadDirectory(String.Empty)
			SetCommandGui(True)
		End Sub

		''' <summary>
		''' Event that is fired when a transfer is in progress
		''' </summary>
		Private Sub FtpTransferProgressChangedEvent(ByVal sender As Object, ByVal e As TransferProgressChangedEventArgs)
			Try
				SetStatus("Transfer [" & _sftp.CurrentFileBeingTransferred & " in " & _sftp.CurrentDirectoryBeingTransferred & "]: " & e.ProgressPercentage & "% (" & e.BytesTransferred & " of " & e.TotalBytesToTransfer & " bytes)")
			Catch myException As Exception
				ShowError(myException)
				Application.Exit()
			End Try
		End Sub

		''' <summary>
		''' Fired when form loads: initializes the FTPTransfer class and sets the event handlers
		''' </summary>
		Private Sub Demo_Load(ByVal sender As Object, ByVal e As EventArgs) Handles Me.Load
			Try
				_sftp = New SFTP()
				_sftp.EnableLogging()
				AddHandler _sftp.OverallProgressEvent, AddressOf FtpTransferProgressChangedEvent
				AddHandler _sftp.FileProgressEvent, AddressOf FtpTransferProgressChangedEvent
				AddHandler _sftp.TransferCompleteEvent, AddressOf FtpTransferCompleted
			Catch myException As Exception
				ShowError(myException)
				Application.Exit()
			End Try
		End Sub

		''' <summary>
		''' Either enables or disables the connection GUI
		''' </summary>
		Private Sub SetConnectGui(ByVal enabled As Boolean)
			txtHost.Enabled = enabled
			txtLogin.Enabled = enabled
			txtPass.Enabled = enabled
			txtPort.Enabled = enabled
			txtPrivateKeyFile.Enabled = enabled
			txtPassphrase.Enabled = enabled
			chkOverWrite.Enabled = enabled
			chkReadOnly.Enabled = enabled
		End Sub

		''' <summary>
		''' Either enables or disables the FTP command GUI
		''' </summary>
		Private Sub SetCommandGui(ByVal enabled As Boolean)
			' this gets called from an event, so make sure this is on the right thread
			If InvokeRequired Then
				' this is not on the right thread, so invoke it on the correct one
				Dim setCommandGuiProc As SetCommandGuiDelegate = AddressOf SetCommandGui
				Invoke(setCommandGuiProc, New Object() { enabled })
				Return
			End If

			btnCreateDirectory.Enabled = enabled
			btnDeleteDirectory.Enabled = enabled
			btnDownloadDir.Enabled = enabled
			btnUploadDir.Enabled = enabled

			btnDownloadFile.Enabled = enabled
			btnUploadFile.Enabled = enabled
			btnDeleteFile.Enabled = enabled
			btnRenameFile.Enabled = enabled
		End Sub

		''' <summary>
		''' Set the status bar text
		''' </summary>
		Private Sub SetStatus(ByVal s As String)
			' this gets called from an event, so make sure this is on the right thread
			If status.InvokeRequired Then
				' this is not on the right thread, so invoke it on the correct one
				Dim setStatusProc As SetStatusDelegate = AddressOf SetStatus
				status.Invoke(setStatusProc, New Object() { s })
				Return
			End If
			status.Items("Label").Text = s
			status.Refresh()
		End Sub

		''' <summary>
		''' Set the status bar text to the status code of an FTP Request
		''' </summary>
		Private Sub SetCode(ByVal response As FtpResponse)
			status.Items("FTPCode").Text = "FTP Status: " & response.Text
			status.Refresh()
		End Sub

		''' <summary>
		''' Load a directory listing into the GUI
		''' </summary>
		''' <param name="dirName">The directory to load, pass string.empty to reload the current directory</param>
		Private Sub LoadDirectory(ByVal dirName As String)
			SetStatus("Loading directory listing...")

			' this gets called from an event, so make sure this is on the right thread
			If treeDir.InvokeRequired Then
				' this is not on the right thread, so invoke it on the correct one
				Dim loadDirectoryProc As LoadDirectoryDelegate = AddressOf LoadDirectory
				Invoke(loadDirectoryProc, New Object() { dirName })
				Return
			End If


			If dirName.Length > 0 Then
				_sftp.CurrentDirectory = _sftp.CurrentDirectory & dirName
			End If

			_files = _sftp.GetDirectoryListing()
			Dim n As TreeNode

			treeDir.Nodes.Clear()
			treeFiles.Nodes.Clear()
			Dim upDir As Boolean = False

			For Each file As FTPFileInfo In _files
				If file.IsDirectory Then
					treeDir.Nodes.Add(file.FileName)
					If file.FileName = ".." OrElse file.FileName = "." Then
						upDir = True
					End If
				Else
					n = treeFiles.Nodes.Add(file.FileName)
					n.ToolTipText = "Modified: " & file.Modified & ControlChars.Lf & "Size: " & file.SizeInBytes
				End If
			Next file

			If Not upDir Then
				treeDir.Nodes.Insert(0, "..")
			End If

			SetStatus("")
		End Sub

		''' <summary>
		''' Event that is fired when a the connect button is clicked, set the FTPTransfer properties and load the default directory
		''' </summary>
		Private Sub btnConnect_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnConnect.Click
			Try
				btnConnect.Enabled = False
				If btnConnect.Text = "Go" Then
					SetStatus("Logging in...")
					SetConnectGui(False)
					_sftp.HostAddress = txtHost.Text
					_sftp.UserName = txtLogin.Text
					_sftp.Password = txtPass.Text
					_sftp.Port = Int32.Parse(txtPort.Text)
					_sftp.OverwriteFiles = chkOverWrite.Checked
					_sftp.OverwriteReadOnlyFiles = chkReadOnly.Checked
					_sftp.SshKeyFile = txtPrivateKeyFile.Text
					_sftp.SshPassphrase = txtPassphrase.Text
					_sftp.Connect()

					LoadDirectory(String.Empty)
					btnConnect.Text = "Dispose"
					SetCommandGui(True)
				Else
					_sftp.Disconnect()
					_sftp.Dispose()
					treeDir.Nodes.Clear()
					treeFiles.Nodes.Clear()
					btnConnect.Text = "Go"
					SetConnectGui(True)
					SetCommandGui(False)
				End If
				btnConnect.Enabled = True
			Catch myException As Exception
				ShowError(myException)
				Application.Exit()
			End Try
		End Sub

		''' <summary>
		''' Delete a selected directory
		''' </summary>
		Private Sub btnDeleteDirectory_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnDeleteDirectory.Click
			Try
				If treeDir.SelectedNode IsNot Nothing Then
				If treeDir.SelectedNode.Text <> "." AndAlso treeDir.SelectedNode.Text <> ".." Then
					SetStatus("Deleting directory...")
					If _sftp.DeleteDirectory(treeDir.SelectedNode.Text) Then
						SetStatus("")
						LoadDirectory(String.Empty)
						SetCode(_sftp.LastFtpResponse)
					Else
						SetStatus("Directory deletion failed...")
					End If
				End If
			Else
				MessageBox.Show("Please select a directory to delete!")
			End If
			Catch myException As Exception
				ShowError(myException)
				Application.Exit()
			End Try
		End Sub

		''' <summary>
		''' Delete a selected file
		''' </summary>
		Private Sub btnDeleteFile_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnDeleteFile.Click
			Try
				If treeFiles.SelectedNode IsNot Nothing Then
					SetStatus("Deleting file...")
					_sftp.DeleteFile(treeFiles.SelectedNode.Text)
					LoadDirectory(String.Empty)
					SetCode(_sftp.LastFtpResponse)
				Else
					MessageBox.Show("Please select a file to delete!")
				End If
			Catch myException As Exception
				ShowError(myException)
				Application.Exit()
			End Try
		End Sub

		''' <summary>
		''' Fired when the directory tree is double clicked, navigate to that directory
		''' </summary>
		Private Sub treeDir_DoubleClick(ByVal sender As Object, ByVal e As EventArgs) Handles treeDir.DoubleClick
			If treeDir.SelectedNode IsNot Nothing Then
				LoadDirectory(treeDir.SelectedNode.Text)
			End If
		End Sub

		''' <summary>
		''' Rename a selected file by placing an edit box on it
		''' </summary>
		Private Sub btnRenameFile_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnRenameFile.Click
			If treeFiles.SelectedNode Is Nothing Then
				MessageBox.Show("Please select a file to rename!")
				Return
			End If

			treeFiles.LabelEdit = True
			If treeFiles.SelectedNode.IsEditing = False Then
				treeFiles.SelectedNode.BeginEdit()
			End If

		End Sub

		''' <summary>
		''' Fired when the user is done editing the file's name, update it
		''' </summary>
		Private Sub treeFiles_AfterLabelEdit(ByVal sender As Object, ByVal e As NodeLabelEditEventArgs) Handles treeFiles.AfterLabelEdit
			Try
				If e.Label IsNot Nothing Then
					If e.Label.Length > 0 Then
						SetStatus("Checking for file existence...")
						If _sftp.FileExists(e.Label) Then
							MessageBox.Show("File already exists!")
							e.CancelEdit = True
						Else
							SetStatus("Renaming file...")
							_sftp.RenameFile(e.Node.Text, e.Label)
							SetCode(_sftp.LastFtpResponse)
						End If

						SetStatus("")
					Else
						MessageBox.Show("File name's cannot be blank!")
						e.CancelEdit = True
					End If
				End If
				treeFiles.LabelEdit = False
			Catch myException As Exception
				ShowError(myException)
				Application.Exit()
			End Try
		End Sub

		''' <summary>
		''' Fired when the user is done editing the new directory's name, create it
		''' </summary>
		Private Sub treeDir_AfterLabelEdit(ByVal sender As Object, ByVal e As NodeLabelEditEventArgs) Handles treeDir.AfterLabelEdit
			Try
				If e.Label IsNot Nothing Then
					If e.Label.Length > 0 Then
						Dim exists As Boolean = False
						For Each n As TreeNode In treeDir.Nodes
							If n.Text = e.Label AndAlso Not n.IsEditing Then
								exists = True
								Exit For
							End If
						Next n

						If exists Then
							e.CancelEdit = True
							e.Node.Remove()
							MessageBox.Show("Directory already exists!")
						Else
							SetStatus("Creating directory...")
							_sftp.CreateDirectory(e.Label)
							SetCode(_sftp.LastFtpResponse)
						End If

						SetStatus("")
					Else
						MessageBox.Show("Directory name cannot be blank!")
						e.CancelEdit = True
					End If
				End If
				treeFiles.LabelEdit = False
			Catch myException As Exception
				ShowError(myException)
				Application.Exit()
			End Try
		End Sub

		''' <summary>
		''' Upload a file
		''' </summary>
		Private Sub btnUploadFile_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnUploadFile.Click
			Try
				If openFileDialog.ShowDialog() = System.Windows.Forms.DialogResult.OK Then
					' check for file existence
					Dim fileName As String = Path.GetFileName(openFileDialog.FileName)
					If Not _sftp.FileExists(fileName) Then
						SetCommandGui(False)
						_sftp.UploadFileAsync(openFileDialog.FileName, fileName)
					Else
						MessageBox.Show("File already exists!")
					End If
				End If
			Catch myException As Exception
				ShowError(myException)
				Application.Exit()
			End Try
		End Sub

		''' <summary>
		''' Download a file
		''' </summary>
		Private Sub btnDownloadFile_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnDownloadFile.Click
			Try
				If treeFiles.SelectedNode Is Nothing Then
					SetCommandGui(False)
					MessageBox.Show("Please select a file to download!")
					Return
				End If

				saveFileDialog.FileName = treeFiles.SelectedNode.Text
				If saveFileDialog.ShowDialog() = System.Windows.Forms.DialogResult.OK Then
					_sftp.DownloadFileAsync(saveFileDialog.FileName, treeFiles.SelectedNode.Text)
				End If
			Catch myException As Exception
				ShowError(myException)
				Application.Exit()
			End Try
		End Sub

		''' <summary>
		''' Begin creating a directory by creating a new edit box for the name
		''' </summary>
		Private Sub btnCreateDirectory_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnCreateDirectory.Click
			treeDir.LabelEdit = True
			Dim n As TreeNode = treeDir.Nodes.Add("New Directory", "New Directory")
			treeDir.SelectedNode = n
			n.BeginEdit()
		End Sub

		''' <summary>
		''' Upload a directory
		''' </summary>
		Private Sub btnUploadDir_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnUploadDir.Click
			Try
				If folderBrowser.ShowDialog() = System.Windows.Forms.DialogResult.OK Then
					SetCommandGui(False)
					Dim d As New DirectoryInfo(folderBrowser.SelectedPath)
					_sftp.UploadDirectoryAsync(folderBrowser.SelectedPath, d.Name)
				End If
			Catch myException As Exception
				ShowError(myException)
				Application.Exit()
			End Try
		End Sub

		''' <summary>
		''' Download a directory
		''' </summary>
		Private Sub btnDownloadDir_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnDownloadDir.Click
			Try
				If treeDir.SelectedNode Is Nothing Then
					SetCommandGui(True)
					MessageBox.Show("Please select a directory to download!")
					Return
				End If

				If folderBrowser.ShowDialog() = System.Windows.Forms.DialogResult.OK Then
					SetCommandGui(False)
					_sftp.DownloadDirectoryAsync(folderBrowser.SelectedPath,treeDir.SelectedNode.Text)
				End If
			Catch myException As Exception
				ShowError(myException)
				Application.Exit()
			End Try
		End Sub

		''' <summary>
		''' Cancel a transfer
		''' </summary>
		Private Sub btnCancel_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnCancel.Click
			Try
				_sftp.Cancel()
			Catch myException As Exception
				ShowError(myException)
				Application.Exit()
			End Try
		End Sub

		Private Sub btnPrivateKeyFile_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnPrivateKeyFile.Click
			If openFileDialog.ShowDialog() = System.Windows.Forms.DialogResult.OK Then
				txtPrivateKeyFile.Text = openFileDialog.FileName
			End If
		End Sub
		#End Region

	End Class
End Namespace